@@ -44,6 +44,27 @@ pub struct SourceOverride {
     build_depends_excludes: Option<Vec<String>>,
 }
 
+impl SourceOverride {
+    pub fn new(
+        section: Option<String>,
+        policy: Option<String>,
+        homepage: Option<String>,
+        vcs_git: Option<String>,
+        vcs_browser: Option<String>,
+        build_depends: Option<Vec<String>>,
+        build_depends_excludes: Option<Vec<String>>,
+    ) -> Self {
+        Self {
+            section,
+            policy,
+            homepage,
+            vcs_git,
+            vcs_browser,
+            build_depends,
+            build_depends_excludes,
+        }
+    }
+}
 #[derive(Deserialize, Debug, Clone, Default)]
 pub struct PackageOverride {
     section: Option<String>,
@@ -55,6 +76,7 @@ pub struct PackageOverride {
     provides: Option<Vec<String>>,
     extra_lines: Option<Vec<String>>,
     test_is_broken: Option<bool>,
+    test_architecture: Option<Vec<String>>,
     test_depends: Option<Vec<String>>,
 }
 
@@ -204,6 +226,10 @@ impl Config {
         self.with_package(key, |pkg| pkg.test_is_broken)
     }
 
+    pub fn package_test_architecture(&self, key: PackageKey) -> Option<&Vec<String>> {
+        self.with_package(key, |pkg| pkg.test_architecture.as_ref())
+    }
+
     pub fn package_test_depends(&self, key: PackageKey) -> Option<&Vec<String>> {
         self.with_package(key, |pkg| pkg.test_depends.as_ref())
     }
