@@ -18,6 +18,7 @@ use cargo::{
 use filetime::{set_file_times, FileTime};
 use flate2::read::GzDecoder;
 use glob::Pattern;
+use itertools::Itertools;
 use regex::Regex;
 use semver::Version;
 use tar::Archive;
@@ -99,6 +100,50 @@ pub fn crate_name_ver_to_dep(crate_name: &str, version: Option<&str>) -> Result<
     Dependency::parse(crate_name, version.as_deref(), source_id)
 }
 
+// attempt to map back a version requirement to a version that can be used as last resort
+// fallback in case all versions satisfying the requirement are yanked
+fn ver_req_to_ver(dep: &Dependency) -> Option<Version> {
+    match dep.version_req() {
+        // if any version would satisfy the dep, but all of them are yanked, we probably don't want
+        // to use this crate anyway ;)
+        cargo::util::OptVersionReq::Any => None,
+        // a regular version requirement is only useful if it has a single element
+        cargo::util::OptVersionReq::Req(req) if req.comparators.len() == 1 => {
+            let comp = req.comparators.first().unwrap();
+            match comp.op {
+                // these are all satisfied by the full version, if one is set
+                semver::Op::Exact
+                | semver::Op::GreaterEq
+                | semver::Op::LessEq
+                | semver::Op::Tilde
+                | semver::Op::Caret => {
+                    if let Some(minor) = comp.minor {
+                        // if the requirement comes with a full version extract it
+                        comp.patch.map(|patch| Version {
+                            major: comp.major,
+                            minor,
+                            patch,
+                            pre: comp.pre.clone(),
+                            build: semver::BuildMetadata::default(),
+                        })
+                    } else {
+                        // we'd have to guess here.
+                        None
+                    }
+                }
+                // Greater, Less or Wildcard all require guessing as well
+                _ => None,
+            }
+        }
+        // requirements with multiple constraints
+        cargo::util::OptVersionReq::Req(_) => None,
+        // locked requirements contain an exact version
+        cargo::util::OptVersionReq::Locked(ver, _) => Some(ver.clone()),
+        // precise requirements contain an exact version
+        cargo::util::OptVersionReq::Precise(ver, _) => Some(ver.clone()),
+    }
+}
+
 pub fn show_dep(dep: &Dependency) -> String {
     format!("{} {}", dep.package_name(), dep.version_req())
 }
@@ -171,17 +216,10 @@ impl CrateInfo {
                     targets: Vec::new(),
                     to_package: Packages::Default,
                     keep_going: false,
+                    reg_or_index: None,
                 };
+                ops::package(&workspace, &opts)?;
 
-                // as of cargo 0.41 this returns a FileLock with a temp path, instead of the one
-                // it got renamed to
-                if ops::package(&workspace, &opts)?.is_none() {
-                    return Err(format_err!(
-                        "Failed to assemble crate file for local crate {} at {}\n",
-                        crate_name,
-                        crate_path.display()
-                    ));
-                }
                 let filename = format!("{}-{}.crate", crate_name, package_id.version());
                 workspace
                     .target_dir()
@@ -238,17 +276,32 @@ impl CrateInfo {
             source_id.url().host_str().unwrap_or(""),
             hash(&source_id).swap_bytes()
         );
-        let get_package_info = |context: &GlobalContext| -> Result<_> {
+        let get_package_info = |context: &GlobalContext,
+                                possibly_yanked_ver: Option<&Version>|
+         -> Result<_> {
             let lock = context.acquire_package_cache_lock(CacheLockMode::DownloadExclusive)?;
             let mut registry =
                 PackageRegistry::new_with_source_config(context, SourceConfigMap::new(context)?)?;
+
+            // if we have some exact version that should satisfy our dependency requirements, and
+            // we are on our third attempt of retrieving the crate info from the registry, add this
+            // version to the yanked whitelist as a last resort
+            if let Some(ver) = possibly_yanked_ver {
+                let pkgids = vec![PackageId::new(
+                    dependency.package_name(),
+                    ver.clone(),
+                    dependency.source_id(),
+                )];
+                debcargo_warn!(
+                    "Adding {} to yanked whitelist as last resort..",
+                    pkgids.first().unwrap()
+                );
+                registry.add_to_yanked_whitelist(pkgids.into_iter());
+            }
             registry.lock_patches();
             let summaries = fetch_candidates(&mut registry, dependency)?;
             drop(lock);
-            let pkgids = summaries
-                .into_iter()
-                .map(|s| s.package_id())
-                .collect::<Vec<_>>();
+            let pkgids = summaries.into_iter().map(|s| s.package_id()).collect_vec();
             let pkgid = pkgids.iter().max().ok_or_else(|| {
                 format_err!(
                     concat!(
@@ -260,6 +313,7 @@ impl CrateInfo {
             })?;
             let pkgset = registry.get(pkgids.as_slice())?;
             let package = pkgset.get_one(*pkgid)?;
+
             let manifest = package.manifest();
             for f in dependency.features() {
                 // apparently, if offline is set then cargo sometimes selects
@@ -287,8 +341,16 @@ impl CrateInfo {
         };
         // if update is false but the user never downloaded the crate then the
         // first call will error; re-try with online in that case
-        let (package, manifest, crate_file) =
-            get_package_info(&context).or_else(|_| get_package_info(&GlobalContext::default()?))?;
+        let (package, manifest, crate_file) = get_package_info(&context, None)
+            .or_else(|_| get_package_info(&GlobalContext::default()?, None))
+            .or_else(|err| {
+                let ver = ver_req_to_ver(dependency);
+                if ver.is_some() {
+                    get_package_info(&GlobalContext::default()?, ver.as_ref())
+                } else {
+                    Err(err)
+                }
+            })?;
 
         Ok(CrateInfo {
             package,
@@ -578,7 +640,7 @@ impl CrateInfo {
         self.includes = includes.cloned().unwrap_or_default();
     }
 
-    pub fn filter_path(&self, path: &Path) -> ::std::result::Result<bool, String> {
+    pub fn filter_path(&self, path: &Path) -> std::result::Result<bool, String> {
         let top_level = path
             .ancestors()
             .find(|p| p.parent() == Some(Path::new("")))
@@ -695,13 +757,13 @@ impl CrateInfo {
         if !actual_toml.contains("AUTOMATICALLY GENERATED BY CARGO") {
             // This logic should only fire for old crates, and that's what the
             // if-conditional is supposed to check; modern versions of cargo
-            // already do this before uploading the crate and we shouldn't need
+            // already do this before uploading the crate, and we shouldn't need
             // to handle it specially.
             log::debug!("Cargo.toml not canonicalized..");
             let orig_toml = actual_toml.clone();
 
             // Some old uncanonicalized Cargo.toml files contain invalid relative references to
-            // license or readme files that newer cargo doesn't accept anymore.. attempt to monkey
+            // license or readme files that newer cargo doesn't accept anymore. attempt to monkey
             // patch them if possible
             let mut check_reference = |name, value: Option<&str>| {
                 if let Some(referenced_path) = value {
@@ -748,12 +810,12 @@ impl CrateInfo {
                 verify: false,
                 jobs: None,
                 keep_going: false,
-                to_package: cargo::ops::Packages::Default,
+                to_package: Packages::Default,
                 targets: Vec::new(),
                 cli_features: CliFeatures::new_all(true),
+                reg_or_index: None,
             };
-            let res = cargo::ops::package(&ws, &opts)?;
-            let files = res.ok_or_else(|| format_err!("Failed to canonicalize"))?;
+            let files = ops::package(&ws, &opts)?;
             let file = files
                 .first()
                 .ok_or_else(|| format_err!("No canonicalized archives found.."))?;
@@ -771,7 +833,7 @@ impl CrateInfo {
                 }
             }
             fs::write(path.join("Cargo.toml.orig"), orig_toml.as_bytes())?;
-            std::fs::remove_dir_all(path.join("target"))?;
+            fs::remove_dir_all(path.join("target"))?;
             source_modified = true;
             // avoid lintian errors about package-contains-ancient-file
             // TODO: do we want to do this for unmodified tarballs? it would
