@@ -42,6 +42,12 @@ pub struct Description {
     pub suffix: String,
 }
 
+impl Description {
+    pub fn new(prefix: String, suffix: String) -> Self {
+        Self { prefix, suffix }
+    }
+}
+
 pub struct PkgTest {
     name: String,
     crate_name: String,
@@ -50,6 +56,7 @@ pub struct PkgTest {
     extra_test_args: Vec<String>,
     depends: Vec<String>,
     extra_restricts: Vec<String>,
+    architecture: Vec<String>,
 }
 
 impl fmt::Display for Source {
@@ -149,6 +156,9 @@ impl fmt::Display for PkgTest {
             "Restrictions: allow-stderr, skip-not-installable{}",
             restricts,
         )?;
+        if !self.architecture.is_empty() {
+            writeln!(f, "Architecture: {}", self.architecture.join(" "))?;
+        }
         Ok(())
     }
 }
@@ -314,13 +324,18 @@ impl Package {
             format!("-{}.{}.{}", version.major, version.minor, version.patch),
         ];
         for suffix in version_suffixes.iter() {
+            // don't provide unversioned variants in semver-suffix packages
+            if name_suffix.is_some() && suffix.is_empty() {
+                continue;
+            };
+
             let p = format!("{}{}", basename, suffix);
             provides.push(deb_feature2(&p, feature.unwrap_or("")));
             provides.extend(f_provides.iter().map(|f| deb_feature2(&p, f)));
         }
         let provides_self = deb_feature(feature.unwrap_or(""));
         // rust dropped Vec::remove_item for annoying reasons, the below is
-        // an unofficial recommended replacement from the RFC #40062
+        // an unofficialy recommended replacement from the RFC #40062
         let i = provides.iter().position(|x| *x == *provides_self);
         i.map(|i| provides.remove(i));
 
@@ -370,10 +385,14 @@ impl Package {
             description,
             extra_lines: match (name_suffix, feature) {
                 (Some(_), None) => {
-                    let fullpkg = format!("{}-{}", basename, version);
+                    // B+R needs to be set on "real" package, not virtual ones
+                    // constrain by "next" version, so that it is possible to install a newer,
+                    // non-suffixed package at the same time
+                    let mut next_version = version.clone();
+                    next_version.patch += 1;
                     vec![
-                        format!("Replaces: {}", deb_name(&fullpkg)),
-                        format!("Breaks: {}", deb_name(&fullpkg)),
+                        format!("Replaces: {} (<< {}~)", deb_name(basename), next_version),
+                        format!("Breaks: {} (<< {}~)", deb_name(basename), next_version),
                     ]
                 }
                 (_, _) => vec![],
@@ -504,6 +523,7 @@ impl fmt::Display for Description {
 }
 
 impl PkgTest {
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         name: &str,
         crate_name: &str,
@@ -512,6 +532,7 @@ impl PkgTest {
         extra_test_args: Vec<&str>,
         depends: &[String],
         extra_restricts: Vec<&str>,
+        architecture: &[&str],
     ) -> Result<PkgTest> {
         Ok(PkgTest {
             name: name.to_string(),
@@ -521,6 +542,7 @@ impl PkgTest {
             extra_test_args: extra_test_args.iter().map(|x| x.to_string()).collect(),
             depends: depends.to_vec(),
             extra_restricts: extra_restricts.iter().map(|x| x.to_string()).collect(),
+            architecture: architecture.iter().map(|x| x.to_string()).collect(),
         })
     }
 }
