@@ -47,19 +47,19 @@ impl V {
     }
 }
 
-impl cmp::Ord for V {
+impl Ord for V {
     fn cmp(&self, other: &V) -> cmp::Ordering {
         self.mmp().cmp(&other.mmp())
     }
 }
 
-impl cmp::PartialOrd for V {
+impl PartialOrd for V {
     fn partial_cmp(&self, other: &V) -> Option<cmp::Ordering> {
         Some(self.cmp(other))
     }
 }
 
-impl cmp::PartialEq for V {
+impl PartialEq for V {
     fn eq(&self, other: &V) -> bool {
         self.mmp() == other.mmp()
     }
@@ -102,7 +102,7 @@ impl VRange {
         self
     }
 
-    fn to_deb_or_clause(&self, base: &str, suffix: &str) -> Result<String> {
+    fn to_deb_clause(&self, base: &str, suffix: &str) -> Result<String> {
         use debian::dependency::V::*;
         match (&self.ge, &self.lt) {
             (None, None) => Ok(format!("{}{}", base, suffix)),
@@ -114,34 +114,48 @@ impl VRange {
                 }
                 let mut ranges = vec![];
                 let (lt_maj, lt_min, lt_pat) = lt.mmp();
-                let (ge_maj, ge_min, ge_pat) = ge.mmp();
-                if ge_maj < lt_maj {
-                    ranges.push((M(ge_maj), Some((true, ge))));
-                    ranges.extend((ge_maj + 1..lt_maj).map(|maj| (M(maj), None)));
-                    ranges.push((M(lt_maj), Some((false, lt))));
+                let (ge_maj, ge_min, _ge_pat) = ge.mmp();
+
+                if ge_maj + 1 == lt_maj && lt_min == 0 && lt_pat == 0 {
+                    // upper bound doesn't restrict lower bound further if we include the major
+                    // part
+                    ranges.push((Some(M(ge_maj)), true, ge));
+                } else if ge_maj < lt_maj {
+                    // different major versions, unversioned package needs to satisfy
+                    ranges.push((None, true, ge));
+                    ranges.push((None, false, lt));
                 } else {
-                    assert!(ge_maj == lt_maj);
-                    if ge_min < lt_min {
-                        ranges.push((MM(ge_maj, ge_min), Some((true, ge))));
-                        ranges.extend((ge_min + 1..lt_min).map(|min| (MM(ge_maj, min), None)));
-                        ranges.push((MM(lt_maj, lt_min), Some((false, lt))));
+                    assert_eq!(ge_maj, lt_maj);
+                    if ge_maj == 0 && ge_min + 1 == lt_min && lt_pat == 0 {
+                        // upper bound doesn't restrict lower bound further if we include 0.X
+                        ranges.push((Some(MM(ge_maj, ge_min)), true, ge));
+                    } else if ge_maj == 0 && ge_min < lt_min {
+                        // different 0.X versions, unversioned package needs to satisfy
+                        ranges.push((None, true, ge));
+                        ranges.push((None, false, lt));
+                    } else if ge_min < lt_min {
+                        // different minor versions within a major version, any package with the
+                        // corresponding major version can potentially satisfy
+                        ranges.push((Some(M(ge_maj)), true, ge));
+                        ranges.push((Some(M(lt_maj)), false, lt));
                     } else {
-                        assert!(ge_min == lt_min);
-                        ranges.push((MMP(ge_maj, ge_min, ge_pat), Some((true, ge))));
-                        ranges.extend(
-                            (ge_pat + 1..lt_pat).map(|pat| (MMP(ge_maj, ge_min, pat), None)),
-                        );
-                        ranges.push((MMP(lt_maj, lt_min, lt_pat), Some((false, lt))));
+                        // just the patch level differs, but both ends restricted
+                        // any package with the corresponding major.minor version can potentially
+                        // satisfy
+                        assert_eq!(ge_min, lt_min);
+                        ranges.push((Some(MM(ge_maj, ge_min)), true, ge));
+                        ranges.push((Some(MM(lt_maj, lt_min)), false, lt));
                     }
                 };
-                // reverse the order so higher versions go first
-                // this helps sbuild find build-deps, it does not resolve alternatives by default
+                // unversioned package name is only provided by the non semver-suffixed packages
+                // if a range is only satisfiable by semver-suffixed variants in the archive, it
+                // needs to be collapsed/reduced accordingly
                 Ok(ranges
                     .iter()
-                    .rev()
-                    .filter_map(|(ver, cons)| match cons {
-                        None => Some(format!("{}-{}{}", base, ver, suffix)),
-                        Some((true, c)) => {
+                    .filter_map(|(ver, greater, cons)| match (ver, greater, cons) {
+                        (None, true, c) => Some(format!("{}{} (>= {}-~~)", base, suffix, c)),
+                        (None, false, c) => Some(format!("{}{} (<< {}-~~)", base, suffix, c)),
+                        (Some(ver), true, c) => {
                             if c == &ver {
                                 // A-x >= x is redundant, drop the >=
                                 Some(format!("{}-{}{}", base, ver, suffix))
@@ -149,7 +163,7 @@ impl VRange {
                                 Some(format!("{}-{}{} (>= {}-~~)", base, ver, suffix, c))
                             }
                         }
-                        Some((false, c)) => {
+                        (Some(ver), false, c) => {
                             if c == &ver {
                                 // A-x << x is unsatisfiable, drop it
                                 None
@@ -158,7 +172,7 @@ impl VRange {
                             }
                         }
                     })
-                    .join(" | "))
+                    .join(", "))
             }
         }
     }
@@ -271,7 +285,7 @@ fn generate_version_constraints(
 
         (_, _) => {
             // https://github.com/dtolnay/semver/issues/262
-            panic!("Op is non-exhuastive for some reason");
+            panic!("Op is non-exhaustive for some reason");
         }
     }
 
@@ -279,7 +293,7 @@ fn generate_version_constraints(
 }
 
 /// Translates a Cargo dependency into a Debian package dependency.
-pub fn deb_dep(config: &Config, dep: &Dependency) -> Result<Vec<String>> // result is a AND-clause
+pub fn deb_dep(config: &Config, dep: &Dependency) -> Result<Vec<String>> // result is an AND-clause
 {
     let dep_dashed = base_deb_name(&dep.package_name());
     let mut suffixes = Vec::new();
@@ -292,7 +306,7 @@ pub fn deb_dep(config: &Config, dep: &Dependency) -> Result<Vec<String>> // resu
     if suffixes.is_empty() {
         suffixes.push("-dev".to_string());
     }
-    let req = semver::VersionReq::parse(&dep.version_req().to_string()).unwrap();
+    let req = semver::VersionReq::parse(&dep.version_req().to_string())?;
     let mut deps = Vec::new();
     for suffix in suffixes {
         let base = format!("{}-{}", Package::pkg_prefix(), dep_dashed);
@@ -301,12 +315,12 @@ pub fn deb_dep(config: &Config, dep: &Dependency) -> Result<Vec<String>> // resu
             let op = coerce_unacceptable_predicate(dep, p, config.allow_prerelease_deps)?;
             generate_version_constraints(&mut vr, dep, p, op)?;
         }
-        deps.push(vr.to_deb_or_clause(&base, &suffix)?);
+        deps.push(vr.to_deb_clause(&base, &suffix)?);
     }
     Ok(deps)
 }
 
-pub fn deb_deps(config: &Config, cdeps: &[Dependency]) -> Result<Vec<String>> // result is a AND-clause
+pub fn deb_deps(config: &Config, cdeps: &[Dependency]) -> Result<Vec<String>> // result is an AND-clause
 {
     let mut deps = Vec::new();
     for dep in cdeps {
@@ -324,3 +338,6 @@ pub fn deb_dep_add_nocheck(x: &str) -> String {
         .trim_end()
         .to_string()
 }
+
+#[cfg(test)]
+mod tests;
