@@ -1,6 +1,7 @@
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
 use std::fs;
 use std::io::{self, ErrorKind, Read, Seek, Write as IoWrite};
+use std::ops::Deref;
 #[cfg(unix)]
 use std::os::unix::fs::PermissionsExt;
 use std::path::{Path, PathBuf};
@@ -12,6 +13,7 @@ use chrono::{self, Datelike};
 use flate2::read::GzDecoder;
 use flate2::write::GzEncoder;
 use flate2::Compression;
+use itertools::Itertools;
 use regex::Regex;
 use tar::{Archive, Builder};
 use tempfile;
@@ -34,13 +36,13 @@ mod dependency;
 
 pub struct DebInfo {
     upstream_name: String,
-    /// Debian package name without rust- prefix or any semver suffix
+    /// Debian package name without `rust-` prefix or any semver suffix
     base_package_name: String,
     /// Package name suffix after the base package name.
     /// Some implies semver_suffix, i.e. Some("") is different from None
     name_suffix: Option<String>,
     uscan_version_pattern: Option<String>,
-    /// Debian package name without rust- prefix
+    /// Debian package name without `rust-` prefix
     package_name: String,
     deb_upstream_version: String,
     debcargo_version: String,
@@ -169,7 +171,7 @@ pub fn prepare_orig_tarball(
                     Err(e) => debcargo_bail!(e),
                     Ok(r) => {
                         if !r {
-                            new_archive.append(&entry.header().clone(), entry)?;
+                            new_archive.append_data(&mut entry.header().clone(), path, entry)?;
                         } else {
                             writeln!(
                                 io::stderr(),
@@ -211,7 +213,7 @@ pub fn apply_overlay_and_patches(
                 );
             }
         }
-        copy_tree(p.as_path(), tempdir.path()).unwrap();
+        copy_tree(p.as_path(), tempdir.path())?;
     }
     if tempdir.path().join("control").exists() {
         debcargo_warn!(
@@ -219,8 +221,9 @@ pub fn apply_overlay_and_patches(
 it's a maintenance burden. Use debcargo.toml instead."
         )
     }
+    // apply patches to Cargo.toml in case they exist, and re-read it
     if tempdir.path().join("patches").join("series").exists() {
-        // apply patches to Cargo.toml in case they exist, and re-read it
+        debcargo_info!("applying patches..");
         let output_dir = &fs::canonicalize(output_dir)?;
         let stderr = || {
             // create a new owned handle to stderr
@@ -229,15 +232,32 @@ it's a maintenance burden. Use debcargo.toml instead."
                 .open("/dev/stderr")
                 .unwrap()
         };
-        expect_success(
+        // common case, patches might need rebasing!
+        if let Err(err) = expect_success(
             Command::new("quilt")
                 .stdout(stderr())
                 .current_dir(output_dir)
                 .env("QUILT_PATCHES", tempdir.path().join("patches"))
                 .args(["push", "--quiltrc=-", "-a"]),
             "failed to apply patches using quilt",
-        );
+        ) {
+            debcargo_warn!(format!("{err}, attempting cleanup"));
+            let _ = expect_success(
+                Command::new("quilt")
+                    .stdout(stderr())
+                    .current_dir(output_dir)
+                    .env("QUILT_PATCHES", tempdir.path().join("patches"))
+                    .args(["pop", "--quiltrc=-", "-a", "-f"]),
+                "failed to unapply partially applied patches",
+            );
+            fs::remove_dir_all(output_dir.join(".pc"))?;
+            debcargo_bail!("applying patches failed! see above for details..");
+        }
+        debcargo_info!("reloading Cargo.toml..");
         crate_info.replace_manifest(&output_dir.join("Cargo.toml"))?;
+
+        // this should never fail!
+        debcargo_info!("unapplying patches again..");
         expect_success(
             Command::new("quilt")
                 .stdout(stderr())
@@ -245,7 +265,8 @@ it's a maintenance burden. Use debcargo.toml instead."
                 .env("QUILT_PATCHES", tempdir.path().join("patches"))
                 .args(["pop", "--quiltrc=-", "-a"]),
             "failed to unapply patches",
-        );
+        )?;
+        fs::remove_dir_all(output_dir.join(".pc"))?;
     }
     Ok(tempdir)
 }
@@ -466,8 +487,7 @@ echo "debcargo testing: suppressing dh-cargo-built-using";;
         let autogenerated_re = Regex::new(&format!(
             r"^  \* Package (.*) (.*) from {} using debcargo (.*)$",
             &crate_src
-        ))
-        .unwrap();
+        ))?;
 
         // Special-case d/changelog:
         let (mut changelog, changelog_data) = changelog_or_new(tempdir.path())?;
@@ -572,7 +592,7 @@ echo "debcargo testing: suppressing dh-cargo-built-using";;
     Ok(())
 }
 
-fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, std::io::Error>>(
+fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<fs::File, io::Error>>(
     deb_info: &DebInfo,
     crate_info: &CrateInfo,
     config: &Config,
@@ -638,6 +658,23 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
         }
     };
 
+    let test_architecture = |f: &str| {
+        let getparents = |f: &str| features_with_deps.get(f).map(|(d, _)| d);
+        let feature_get_test_architecture =
+            |f: &str| config.package_test_architecture(PackageKey::feature(f));
+        match get_transitive_val(&getparents, &feature_get_test_architecture, f) {
+            Err((k, vv)) => debcargo_bail!(
+                "{} {}: {}: {:?}",
+                "error trying to recursively determine test_architecture for",
+                k,
+                "dependencies have inconsistent config values",
+                vv
+            ),
+            Ok(Some(v)) if v.is_empty() => Ok(None), // allow resetting via explicit empty list
+            Ok(other) => Ok(other),
+        }
+    };
+
     let build_deps = {
         let rustc = rustc_dep(&crate_info.rust_version(), true);
         let build_deps = ["debhelper-compat (= 13)", "dh-sequence-cargo"]
@@ -673,11 +710,19 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
         .into_iter()
         .chain(dev_depends)
         .collect();
+
+    // prefer Cargo.toml homepage, fallback to Cargo.toml repository
+    let homepage = meta
+        .homepage
+        .as_deref()
+        .or(meta.repository.as_deref())
+        .unwrap_or("");
+
     let mut source = Source::new(
         base_pkgname,
         name_suffix,
         crate_name,
-        meta.homepage.as_deref().unwrap_or(""),
+        homepage,
         lib,
         maintainer.to_string(),
         uploaders.iter().map(|s| s.to_string()).collect(),
@@ -716,6 +761,21 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
                 .iter()
                 .any(|f| test_is_marked_broken(f).unwrap_or(false)),
         };
+        let all_features_test_arch = match test_architecture("@")? {
+            Some(v) => v.to_owned(),
+            None => all_features
+                .iter()
+                .fold(HashSet::new(), |mut set, f| {
+                    if let Ok(Some(arch)) = test_architecture(f) {
+                        set.extend(arch.to_owned());
+                    }
+                    set
+                })
+                .into_iter()
+                .collect_vec(),
+        };
+        let all_features_test_arch: Vec<&str> =
+            all_features_test_arch.iter().map(AsRef::as_ref).collect();
         let all_features_test_depends =
             generate_test_dependencies("@", &all_features, config, &test_deps);
         let mut testctl = io::BufWriter::new(file("tests/control")?);
@@ -734,6 +794,7 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
                 } else {
                     vec![]
                 },
+                all_features_test_arch.deref(),
             )?
         )?;
 
@@ -947,6 +1008,11 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
 
                 // deps
                 let test_depends = generate_test_dependencies(f, &feature_deps, config, &test_deps);
+                let test_arch = match test_architecture(f)? {
+                    Some(v) => v.to_owned(),
+                    None => Vec::new(),
+                };
+                let test_arch: Vec<&str> = test_arch.iter().map(AsRef::as_ref).collect();
                 let pkgtest = PkgTest::new(
                     package.name(),
                     crate_name,
@@ -959,6 +1025,7 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
                     } else {
                         vec![]
                     },
+                    test_arch.deref(),
                 )?;
                 write!(testctl, "\n{}", pkgtest)?;
             }
@@ -1059,7 +1126,7 @@ fn reduce_provides(
     mut features_with_deps: CrateDepInfo,
 ) -> (BTreeMap<&'static str, Vec<&'static str>>, CrateDepInfo) {
     // If any features have duplicate dependencies, deduplicate them by
-    // making all of the subsequent ones depend on the first one.
+    // making all the subsequent ones depend on the first one.
     let mut features_rev_deps = HashMap::new();
     for (&f, dep) in features_with_deps.iter() {
         if !features_rev_deps.contains_key(dep) {
@@ -1127,37 +1194,6 @@ fn rustc_dep(min_ver: &Option<String>, native: bool) -> String {
     }
 }
 
-#[cfg(test)]
-mod test {
-    use super::rustc_dep;
-
-    #[test]
-    fn rustc_dep_includes_minver() {
-        assert_eq!(
-            "rustc:native (>= 1.65)",
-            rustc_dep(&Some("1.65".to_string()), true)
-        );
-    }
-
-    #[test]
-    fn rustc_dep_excludes_minver() {
-        assert_eq!("rustc:native", rustc_dep(&None, true));
-    }
-
-    #[test]
-    fn rustc_dep_includes_minver_autopkgtest() {
-        assert_eq!(
-            "rustc (>= 1.65)",
-            rustc_dep(&Some("1.65".to_string()), false)
-        );
-    }
-
-    #[test]
-    fn rustc_dep_excludes_minver_autopkgtest() {
-        assert_eq!("rustc", rustc_dep(&None, false));
-    }
-}
-
 fn changelog_or_new(tempdir: &Path) -> Result<(fs::File, String)> {
     let mut changelog = fs::OpenOptions::new()
         .read(true)
@@ -1189,3 +1225,34 @@ fn changelog_first_last(tempdir: &Path) -> Result<(i32, i32)> {
         Err(format_err!("changelog had no entries"))
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::rustc_dep;
+
+    #[test]
+    fn rustc_dep_includes_minver() {
+        assert_eq!(
+            "rustc:native (>= 1.65)",
+            rustc_dep(&Some("1.65".to_string()), true)
+        );
+    }
+
+    #[test]
+    fn rustc_dep_excludes_minver() {
+        assert_eq!("rustc:native", rustc_dep(&None, true));
+    }
+
+    #[test]
+    fn rustc_dep_includes_minver_autopkgtest() {
+        assert_eq!(
+            "rustc (>= 1.65)",
+            rustc_dep(&Some("1.65".to_string()), false)
+        );
+    }
+
+    #[test]
+    fn rustc_dep_excludes_minver_autopkgtest() {
+        assert_eq!("rustc", rustc_dep(&None, false));
+    }
+}
